/************************************************************************/
/*                                                                      */
/*      DosOut -- Format search output using DOS-specific calls         */
/*                                                                      */
/*      In the interests of keeping the code portability high, the      */
/*      core files have been carefully cut to use only standard         */
/*      ANSI C calls.  However, some special effects require            */
/*      non-standard calls, such as setting character colour            */
/*      attributes.  This is a non-portable module to provide           */
/*      match highlighting output.                                      */
/*                                                                      */
/*      Copyright (C) Grouse Software 1995-7.  All rights reserved.     */
/*      Written for Grouse by behoffski (Brenton Hoff).                 */
/*                                                                      */
/*      Free software: no warranty; use anywhere is ok; spread the      */
/*      sources; note any mods; share variations and derivatives        */
/*      (including sending to behoffski@grouse.com.au).                 */
/*                                                                      */
/************************************************************************/

#ifndef DOSOUT_H
#define DOSOUT_H

#include <compdef.h>
#include "matcheng.h"

#ifdef __cplusplus
extern "C" {
#endif /*__cplusplus*/

/************************************************************************/
/*                                                                      */
/*      Init -- Prepare module for operation                            */
/*                                                                      */
/************************************************************************/
void
DosOut_Init(void);

/************************************************************************/
/*                                                                      */
/*      Display -- Report line and include details as requested         */
/*                                                                      */
/************************************************************************/
BOOL
DosOut_Display(MATCHENG_DETAILS *pDetails);

/************************************************************************/
/*                                                                      */
/*      DisplayHighlighted -- Show line, including highlighting match   */
/*                                                                      */
/************************************************************************/
BOOL
DosOut_DisplayHighlighted(MATCHENG_DETAILS *pDetails);

/************************************************************************/
/*                                                                      */
/*      putsBinary -- Write text but don't allow special characters     */
/*                                                                      */
/************************************************************************/
void
DosOut_putsBinary(CHAR *pStr);

/************************************************************************/
/*                                                                      */
/*      FilenameOut -- Write filename and/or match count summary        */
/*                                                                      */
/************************************************************************/
BOOL
DosOut_FilenameOut(MATCHENG_DETAILS *pDetails);

#ifdef __cplusplus
}
#endif /*__cplusplus*/

#endif /*DOSOUT_H*/
