NAME
        ggrep -- Print lines matching a pattern (using Grouse search)

SYNOPSIS
        ggrep [-bcdDFGhHilLMnOvVwx] ([-e] regexp | -f file) [file(s) | -]

DESCRIPTION
        Ggrep searches the specified files for the specified pattern,
        and by default prints the lines matching the pattern.  The
        files can include wildcards (e.g. e*95.doc).  Stdin is
        searched if the file is not specified or is "-".

        The pattern is usually a command line argument, but can be
        read from the first line of a file using -f if the pattern
        contains things magical to the operating system (e.g. ">").

        Warning: The program has NOT been exhaustively tested.  Use
        at your own risk.

OPTIONS
        -b       Show byte offset of selected lines from start of file
        -c       Count the lines matching the pattern (or nonmatching if -v)
        -d       Include subdirectories in search
        -D       Display state table contents for debugging
        -e expr  Use expr as the pattern (useful if RE starts with "-")
        -F       Use pattern as a string (no special RE meanings)
        -f file  Use first line of file as pattern specification
        -G       Interpret pattern as a basic regular expression
        -h       Suppress filename output when searching multiple files
        -H       Highlight matching text within each line
        -i       Ignore case in pattern and in files
        -l       Print filename where at least one line is selected
        -L       Print filename where no lines are selected
        -M<char> Mark match by adding <char> delimiters
        -O<codes> Enable/disable optimisations (default: all enabled):
                   A/a Approximate match
                   B/b Buffer search
                   E/e Easiest first
                   S/s Skip bytes
        -n       Show line number of selected lines
        -v       Invert match sense: select lines not matching pattern
        -V       Display program version information
        -w       Only match if RE matches an exact word using [A-Za-z0-9_]
        -x       Only match if RE matches entire line

        Options -O, -M and -D (OMD?!) are intended for debug/test use,
        so don't depend too heavily upon them as they'll probably
        change in future releases.  The D and O options are very useful
        for observing how the state tables for a search are constructed
        and optimised.

REGULAR EXPRESSIONS
        Currently ggrep only supports basic grep-style patterns (and
        accepts -F and -G specifier as per GNU Grep).  The regular
        expression specifiers and their meanings are:

        lit     Any char matches itself (except for specifiers below)
        \lit    Char is a literal, not a specifier (e.g. \.)
        .       Match any character
        ^       Match the empty string at the start of the line
        $       Match the empty string at the end   of the line
        [class]  Match any character specified in class, where:
        [^class] Match any character not       in class, where:
                      aeiou   Literals specify themselves as class members
                      a-z0-9  Use "-" to specify range(s) of characters
                      []a-z]  Place "]" first if a literal in the class
                      [0-9-]  Place "-" last  if a literal in the class
                      [`^~]   Place "^" "un-first" if a literal in the class
        ?       Match previous regular expression 0 or 1 times
        *       Match previous regular expression 0 or more times
        +       Match previous regular expression 1 or more times

RETURN CODES
        Ggrep returns 2 if there were any errors in the request,
        including file specifications unable to be globbed.
        Otherwise, it returns 1 if no lines were selected or 0
        if at least one line was selected.

COMPATIBILITY
        Ggrep passes all the Spencer tests for basic regular
        expression searches (1-47, 52-54, 56-61, 66-68, 76, 80-82,
        88-90, 92, 101, 102, 105, 114, 118, 119 and 121), using
        the test batch file from GNU grep.

        LF is used as the line terminator, and CR is ignored if
        present.  This allows the search to work correctly on any
        mixture of DOS- and Unix-format text, even where the
        styles are mingled in the same file.

        Backslashes are not treated properly, as the code leans
        towards behaving the same as GNU Grep rather than bring
        the full joy of Unix to DOS.

PERFORMANCE
        The Grouse finite-state machine architecture is used to
        implement the search.  Many searches will operate faster
        than other programs, especially if case insensitivity,
        classes, and/or iteration are present.  Simple strings
        also perform extremely well.  A few cases perform poorly,
        as the best optimisation isn't found or the effort to
        optimise slows things down.  Try "[qj].*[qj].*[qj]" for
        speed and inverted (-v) searches for slow.

        However, the search uses a nondeterministic (backtracking)
        search engine.  Deterministic engines may run faster in
        some cases.  Thanks to Henry Spencer for pointing this out.

        If you are performing speed tests, place the file(s) to be
        searched onto a RAMdisk or ensure they are present in a
        cache (e.g. SmartDrv).  Otherwise, you will simply measure
        the time to bring the file in from disk, as this is much
        greater than the time taken to search.

        The DOS program does not attempt to overlap reading the
        file in and searching for the string.  This means that it
        typically spends 60-80% of its time waiting for DOS (Godot?).
        An estimate of the file system overhead can be made by
        searching for a string like "xxxxxxxxxxxxxxxxxx" which only
        consumes 3 or 4 cycles per byte to perform the search... so
        nearly all the time is disk.

        File globbing tries to keep both the Unix and the DOS camps
        happy by sorting files and presenting names in Unix format,
        but allowing recursion into subdirectories (without sorting)
        to keep DOS users happy.  File searching is optimised to
        examine each directory only once.

COPYRIGHT
        Ggrep is Copyright (C) 1995-7 Grouse Software.  All rights
        reserved.  The Grouse finite-state architecture and ggrep
        were created by behoffski (Brenton Hoff).

        Beware that as the sources are public, hacked versions
        are quite feasible.  Grouse's FTP archives should be
        reliable (see ref below).

LICENCING
        Ggrep is freeware/mailware/postcardware.  If you find
        ggrep or the Grouse finite-state machine concept valuable,
        please let behoffski know by sending a postcard or an
        e-mail cheerio.

        If you improve the ggrep sources, please consider sharing
        your improvements, including e-mail to Grouse.  If ggrep
        or the Grouse FSA are used in your application, an
        acknowledgement in the documentation would be appreciated.
        And if you really like Grouse's ideas and style, consider
        contributing $20 to Grouse to help fund more public-domain
        software.

                Grouse Software Pty Ltd
                PO Box 195
                Willaston, South Australia, 5118
                Australia

                http://www.grouse.com.au/
                ftp://ftp.grouse.com.au/clients/grouse
                mailto:ggrep@grouse.com.au
