/************************************************************************/
/*                                                                      */
/*      Platform -- Handle platform-specific functions (MS-DOS version) */
/*                                                                      */
/*      This module is part of behoffski's attempt to provide           */
/*      utterly portable sources using link-time or run-time binding    */
/*      instead of using the preprocessor at compile time.              */
/*                                                                      */
/*      Copyright (C) Grouse Software 1995-7.  All rights reserved.     */
/*      Written for Grouse by behoffski (Brenton Hoff).                 */
/*                                                                      */
/*      Free software: no warranty; use anywhere is ok; spread the      */
/*      sources; note any mods; share variations and derivatives        */
/*      (including sending to behoffski@grouse.com.au).                 */
/*                                                                      */
/************************************************************************/

#include "compdef.h"
#include <dos.h>
#include "dosout.h"
#include "glob.h"
#include "ggrep.h"
#include "io.h"
#include "linefile.h"
#include "main.h"
#include "matchc.h"
#include "matchi86.h"
#include "platform.h"
#include "regexp.h"
#include "retable.h"
#include "scanfile.h"
#include <stdio.h>
#include <stdlib.h>

/*Parameters to configure filename globbing*/
#define FILESPECS_MAX                   200
#define FILENAME_BYTES_MAX              50000u

#define USE_C_MATCH                     0

/************************************************************************/
/*                                                                      */
/*      CtrlBrkHandler -- Quiet function to process program break       */
/*                                                                      */
/************************************************************************/
module_scope int
Platform_CtrlBrkHandler(void)
{

        /*Get low-level service routines to clean up and stop*/
        _exit(2);

        /*Line is not reached (but keep compilers happy)*/
        return 0;

} /*CtrlBrkHandler*/

/************************************************************************/
/*                                                                      */
/*      main -- Set up platform-specific things before invoking program */
/*                                                                      */
/************************************************************************/
public_scope int
main(int argc, char **argv)
{
        CHAR *pProgName = "ggrep";

        /*Phase 1: Prepare module internals without referencing others*/

        /*Basic initialisation of each module*/
        DosOut_Init();
        ggrep_Init();
        Glob_Init();
        LineFile_Init();
        RegExp_Init();
        RETable_Init();
        ScanFile_Init();
        MatchC_Init();

        /*Stop program break (Control-Break or CTRL/C) creating a dump*/
        ctrlbrk(Platform_CtrlBrkHandler);

        /*Use _read to read file as it handles pipes better than read (TSC)*/
        /*(To be fair, rewind on a pipe probably isn't clearly defined)*/
        LineFile_ReadFunction(_read);

        /*Phase 2: Notify other modules of existence as appropriate*/
        /*         Also acquire permanent resources*/

        /*Start modules going (get resources and chat to other modules)*/
        if (! Glob_Start(pProgName, FILESPECS_MAX, FILENAME_BYTES_MAX)) {
                fprintf(stderr, "%s: Unable to glob files!\n", pProgName);
                return MAIN_RETURN_FAULT;
        }
        if (! RETable_Start())
        {
                fprintf(stderr, "%s: Unable to start RETable!\n", pProgName);
                return MAIN_RETURN_FAULT;
        }

        /*Set up ScanFile for operation and acquire memory*/
        if (! ScanFile_Start()) {
                /*Sorry, ScanFile/LineFile was unable to prepare*/
                fprintf(stderr, "%s: Not enough memory to scan input!\n",
                                pProgName);
                return MAIN_RETURN_FAULT;
        }

        /*Specify functions to display matches and/or filenames*/
        ScanFile_OutputFunctions(DosOut_Display,
                                 DosOut_DisplayHighlighted,
                                 DosOut_FilenameOut);

        /*Select function to perform match operation*/
        ScanFile_MatchFunction(MatchI86_Match);

        /*To use C version of search engine (for debug or porting), use:*/
        #if USE_C_MATCH
        ScanFile_MatchFunction(MatchC_Match);

        /*Request debug output (note: target requires DEBUGGING to be set)*/
        /*MatchC_Debug(0, MATCHC_D_ACTION);*/
        #endif /*USE_C_MATCH*/

        /*Phase 3: Analyse request and configure everything (see ggrep)*/
        /*Phase 4: Execute request (see ggrep)*/

        /*Invoke main routine and report return code*/
        return ggrep_main(argc, argv);

} /*main*/
