#Test -- Perl script to control regression tests

#This script provides a simple but useful framework for executing
#regression tests for a system.  It is cloned from the test rig
#used to check the Perl distribution.

#behoffski decided to use "te" after reading about Piglet...

$| = 1;
chdir 'te' if -f 'te/test.pl';

if ($ARGV[0] eq '-v') {
	$verbose = 1;
	shift;
}

if (defined($ARGV[0])) {
	@files = <$ARGV[0]/*.te>
} else {
	@files = <*/*.te>;
}

$bad = 0;
foreach $test (@files)
{
	$te = $test;
	s/\n$//o;
	print "$te" . '.' x (24 - length($te));

	#Move to the subdirectory to perform the test
	$dir = 0;
	$dir = $1 if $te =~/(.*)\/(.*)/;
	if ($dir)
	{
		chdir $dir;
		$test = $2;
	}
	

	#Execute the test script
	open(RESULTS, "command/c perl $test |");

	#Check the test results
	$ok = 0;
	$next = 0;
	while (<RESULTS>)
	{
		if ($verbose)
		{
			print $_;
		}
		next if /^#/;


		#Find out how many tests to expect
		if (/^1\.\.([0-9]+)/)
		{
			$max = $1;
			$totmax += $max;
			$files++;
			$next = 1;
			$ok = 1;
			next;
		}
			
		#Check that test is okay and in order
		$next = $1, $ok = 0, last if /^not ok ([0-9]*)/;
		if (/^ok (.*)/ && $1 == $next) {
		    $next = $next + 1;
		} else {
		    $ok = 0;
		}

	}

	chdir ".." if $dir;

	if ($ok && --$next == $max) {
		print "ok ($max)\n";
	} else {
		$next++;
		print "FAILED on test $next (of $max)\n";
		$bad++;
		if ($dir eq "base") {
			die "Failed a basic test--cannot continue.\n";
		}
	}
}

#Provide a concise summary at the end of the test run

print "$totmax tests in $files files.\n";

die "Failed 1 test.\n"     if $bad == 1;
die "Failed $bad tests.\n" if $bad;
die "FAILED--no tests were run for some reason.\n" unless $ok;

print "All tests successful.\n";

