/************************************************************************/
/*                                                                      */
/*      CCodec -- C version of table-driven coder/decoder               */
/*                                                                      */
/*      Copyright (C) Grouse Software 1997.  All rights reserved.       */
/*      Written for Grouse by behoffski (Brenton Hoff).                 */
/*                                                                      */
/*      Free software: no warranty; use anywhere is ok; spread the      */
/*      sources; note any mods; share variations and derivatives        */
/*      (including sending to behoffski@grouse.com.au).                 */
/*                                                                      */
/************************************************************************/

#ifndef CCODEC_H
#define CCODEC_H

#include <compdef.h>
#include <hufftree.h>

#ifdef __cplusplus
extern "C" {
#endif /*__cplusplus*/

typedef struct CCodec_TableStruct *CCODEC_TABLE;

/************************************************************************/
/*                                                                      */
/*      Init -- Prepare module for operation                            */
/*                                                                      */
/************************************************************************/
void
CCodec_Init(void);

/************************************************************************/
/*                                                                      */
/*      PrepDecode -- Prepare table-driven encoder given code list      */
/*                                                                      */
/************************************************************************/
BOOL
CCodec_PrepDecode(UINT NrCodes, WORD EndCode,
                                HuffTree_CodeItem *pCodeList,
                                CCODEC_TABLE *pTable);

/************************************************************************/
/*                                                                      */
/*      Decode -- Decode compressed stream using table-driven FSA       */
/*                                                                      */
/************************************************************************/
UINT
CCodec_Decode(CCODEC_TABLE pTab,
              BYTE *pInput, UINT *pNrDecodeBytes, BYTE *pOutput);

/************************************************************************/
/*                                                                      */
/*      Destroy -- Get rid of table-driven coder                        */
/*                                                                      */
/************************************************************************/
void
CCodec_Destroy(CCODEC_TABLE pTab);

#ifdef __cplusplus
}
#endif /*__cplusplus*/

#endif /*CCODEC_H*/
