/************************************************************************/
/*                                                                      */
/*      FileFreq -- Collate charater frequency stats for file           */
/*                                                                      */
/*      Copyright (C) Grouse Software 1997.  All rights reserved.       */
/*      Written for Grouse by behoffski (Brenton Hoff).                 */
/*                                                                      */
/*      Free software: no warranty; use anywhere is ok; spread the      */
/*      sources; note any mods; share variations and derivatives        */
/*      (including sending to behoffski@grouse.com.au).                 */
/*                                                                      */
/************************************************************************/

#include <compdef.h>
#include "filefreq.h"
#include <io.h>
#include <stdio.h>

typedef struct {
        BYTE Buf[16384];
} FILEFREQ_MODULE_CONTEXT;

module_scope FILEFREQ_MODULE_CONTEXT gFileFreq;

/************************************************************************/
/*                                                                      */
/*      Init -- Prepare module for operation                            */
/*                                                                      */
/************************************************************************/
void
FileFreq_Init(void)
{
} /*Init*/

/************************************************************************/
/*                                                                      */
/*      Analyse -- Accumulate frequency stats for specified file        */
/*                                                                      */
/*      This function reads the file data, and increments the           */
/*      frequency counters for each byte found.  The frequencies        */
/*      are not initialised before counting, so that the                */
/*      analysis can span multiple files.  The client must              */
/*      initialise all members of the frequency count array to 0        */
/*      before the first invocation of this routine.                    */
/*                                                                      */
/************************************************************************/
BOOL
FileFreq_Analyse(char *pFilename, UINT32 *pFreqs)
{
        FILE *f;
        BYTE *pBuf;
        UINT NrBytes;

        f = fopen(pFilename, "rb");
        if (f == NULL) {
                /*Unable to open file*/
                return FALSE;
        }

        /*Work through file in a memory buffer*/
        for (;;) {
                /*Get next information of file, if any*/
                NrBytes = _read(fileno(f), gFileFreq.Buf,
                                           sizeof(gFileFreq.Buf));
                if (NrBytes == 0) {
                       /*Finished file*/
                       break;
                }

                /*Work through each byte in buffer*/
                pBuf = gFileFreq.Buf;
                while (NrBytes--) {
                        /*Count this occurrence of the byte*/
                        pFreqs[*pBuf++]++;
                }

        }

        fclose(f);

        return TRUE;

} /*Analyse*/
