/************************************************************************/
/*                                                                      */
/*      HuffTree -- Manipulate Huffman trees                            */
/*                                                                      */
/*      In the same way that "regexp" was a warm-up for REs, this       */
/*      module is a warm-up for trying the Grouse FSA as a way of       */
/*      getting ultra-fast table-driven Huffman coders/decoders.        */
/*                                                                      */
/*      Copyright (C) Grouse Software 1997.  All rights reserved.       */
/*      Written for Grouse by behoffski (Brenton Hoff).                 */
/*                                                                      */
/*      Free software: no warranty; use anywhere is ok; spread the      */
/*      sources; note any mods; share variations and derivatives        */
/*      (including sending to behoffski@grouse.com.au).                 */
/*                                                                      */
/************************************************************************/

#ifndef HUFFTREE_DEF
#define HUFFTREE_DEF

#include "compdef.h"
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif /*__cplusplus*/

/*Extended format for 8 or 16-bit symbols and for encodings up to 64 bits*/

#define HUFFTREE_BIT_BYTES_MAX          8

#define HUFFTREE_FLAG_8BIT_SYM          0x00
#define HUFFTREE_FLAG_16BIT_SYM         BIT7

typedef struct {
        BYTE Flags;
        UINT8 NrBits;
        WORD Symbol;
        BYTE BitBuf[HUFFTREE_BIT_BYTES_MAX];

} HuffTree_CodeItem;

typedef void HuffTree_Tree;

/************************************************************************/
/*                                                                      */
/*      Init -- Prepare module for operation                            */
/*                                                                      */
/************************************************************************/
void
HuffTree_Init(void);

/************************************************************************/
/*                                                                      */
/*      Create -- Form tree, given byte frequencies                     */
/*                                                                      */
/************************************************************************/
BOOL
HuffTree_Create(UINT NrCodes, UINT32 *pFreqs, HuffTree_Tree **ppTree);

/************************************************************************/
/*                                                                      */
/*      Encode -- Encode data using coding tree                         */
/*                                                                      */
/************************************************************************/
void
HuffTree_Encode(HuffTree_CodeItem *pCodeList,
                               UINT NrBytes, WORD EndSymbol,
                               BYTE *pInput,
                               UINT *pNrSquished, BYTE *pSquished);

/************************************************************************/
/*                                                                      */
/*      SymList -- Provide description of tree as list in symbol order  */
/*                                                                      */
/************************************************************************/
UINT
HuffTree_SymList(HuffTree_Tree *pTree, HuffTree_CodeItem *pCodeList);

/************************************************************************/
/*                                                                      */
/*      WriteList -- Save symbol list in file                           */
/*                                                                      */
/************************************************************************/
BOOL
HuffTree_WriteList(FILE *f, UINT NrItems, HuffTree_CodeItem *pCodeList);

/************************************************************************/
/*                                                                      */
/*      ReadList -- Read symbol list from file                          */
/*                                                                      */
/************************************************************************/
BOOL
HuffTree_ReadList(FILE *f, HuffTree_CodeItem *pCodeList,
                                   UINT *pNrCodes);

/************************************************************************/
/*                                                                      */
/*      Show -- Show tree in text format                                */
/*                                                                      */
/************************************************************************/
void
HuffTree_Show(HuffTree_Tree *pTree);

#ifdef __cplusplus
}
#endif /*__cplusplus*/

#endif /*HUFFTREE_DEF*/
