##########################################################################
# MkFSMA -- Make finite-state machine, assembly version
#
# There are 8 bit alignment states, and 10 actions per state.
# Creating the finite-state machine code is mechanical and
# tedious.  This Perl script creates the source in a more
# creative fashion.
#
# Copyright (C) Grouse Software 1995-7.  All rights reserved.
# Written for Grouse by behoffski (Brenton Hoff).
#
# Free software: no warranty; use anywhere is ok; spread the
# sources; note any mods; share variations and derivatives
# (including sending to behoffski@grouse.com.au).

$~ = ROUTINE_START;
write STDOUT;

@PCNextArray = (0x100, 0x1c, 0x38, 0x54, 0x70, 0x8c, 0xa8, 0xc4, 0xe0, 0xe4);
%ShiftSubst = ("shlax", "(*shl ax,n*) db 0C1H,0E0H,",
               "shlbl", "(*shl bl,n*) db 0C0H,0E3H,",
               "shral", "(*shr al,n*) db 0C0H,0E8H,",
               "shrbx", "(*shr bx,n*) db 0C1H,0EBH,");

for ($BitAlign = 0; $BitAlign != 8; $BitAlign++) {

        &Emit("");
        &Emit("(*--------------------------- Bit alignment $BitAlign ---------------------------*)");
        &Emit("");

        for ($Code = 1; $Code < 9; $Code++) {
                &MkEntry($BitAlign, $Code);
        }

        &Emit("public _BitAlign${BitAlign}Finished:");
        &Emit("        jmp     near Finished",   3, 3);

        &Emit("");
        &PadEntry(9);

        $Shift = $BitAlign + 1;

        if ($Shift != 8) {
                &Emit("public _BitAlign${BitAlign}UseGt8:");
                &Emit("        (*pState = &pTab->State[pStateTable[Next8 + NRBYTES]];*)");
                &Emit("        (*Reg = (Reg << 8) | *pInput++; Next8 = LOBYTE(Reg >> $Shift);*)");
                &Emit("        (*Act = pState->Action[Next8]; goto BitAlign$BitAlign; ... switch (Action) {*)");
                &Emit("");

                &Emit("        mov     al,[bx][00100H]", 2, 4);
                &Emit("        mov     dh,dl",           1, 2);
                &Emit("        db 026H; (*es:*) mov dl,[si]",  3, 3);
                &Emit("        mov     bx,dx",           1, 2);
                &Emit("        shr     bx,$Shift",       2, 3);
                &Emit("        mov     bh,al",           1, 2);
                &Emit("        mov     al,[bx]",         1, 2);
                &Emit("        inc     si",              1, 1);
                &Emit("        jmp     ax",              5, 2);

        } else {
                &Emit("public _BitAlign${BitAlign}UseGt8:");
                &Emit("        (*pState = &pTab->State[pStateTable[Next8 + NRBYTES]];*)");
                &Emit("        (*Next8 = LOBYTE(Reg); Reg = (Reg << 8) | *pInput++;*)");
                &Emit("        (*Act = pState->Action[Next8]; goto BitAlign$BitAlign; ... switch (Action) {*)");
                &Emit("");

                &Emit("        mov     bh,[bx][00100H]", 2, 4);
                &Emit("        mov     bl,dl",           1, 2);
                &Emit("        mov     dh,dl",           1, 2);
                &Emit("        db 026H; (*es:*) mov     dl,[si]", 2, 3);
                &Emit("        inc     si",              1, 1);
                &Emit("        mov     al,[bx]",         1, 2);
                &Emit("        jmp     ax",              5, 2);

        }

        &Emit("");
        &PadEntry(0);

}

$~ = ROUTINE_END;
write STDOUT;

exit 1;

##########################################################################
# MkEntry -- Create entry for specified bit alignment and code length
sub MkEntry {
        local($BitAlign, $Code) = @_;
        local($NewAlign);
        local($Align1, $Align2);

        &Emit("public BitAlign${BitAlign}Use$Code:");

        if ($BitAlign < $Code) {
                $NewAlign = $BitAlign + (8 - $Code);
                $Shift   = $NewAlign + 1;
        } else {
                $NewAlign = $BitAlign - $Code;
                $Shift = $NewAlign + 1;
        }

        if (($BitAlign < $Code) && ($Shift != 8)) {
                &Emit("        (**pOutput++ = pStateTable[Next8 + NRBYTES];*)");
                &Emit("        (*pStateTable = &pTab->Table[0][0];*)");
                &Emit("        (*Reg = (Reg << 8) | *pInput++; Next8 = LOBYTE(Reg >> $Shift);*)");
                &Emit("        (*Act = pState->Action[Next8]; goto BitAlign$NewAlign; ... switch (Action) {*)");
                &Emit("");
                &Emit("        mov     al,[bx][00100H]", 2, 4);
                &Emit("        mov     dh,dl",           1, 2);
                &Emit("        db 026H; (*es:*) mov dl,[si]",  2, 3);
                &Emit("        inc     si",              1, 1);
                &Emit("        mov     bx,dx",           1, 2);
                &Emit("        shr     bx,$Shift",       2, 3);
                &Emit("        mov     ah,$NewAlign",    1, 2) if $Code != 8;
                &Emit("        mov     bh,0",            1, 2);
                &Emit("        stosb",                   5, 1);
                &Emit("        mov     al,[bx]",         1, 2);
                &Emit("        jmp     ax",              5, 2);

        } elsif ($BitAlign < $Code) { #$Shift == 8
                &Emit("        (**pOutput++ = pStateTable[Next8 + NRBYTES];*)");
                &Emit("        (*pStateTable = &pTab->Table[0][0];*)");
                &Emit("        (*Next8 = LOBYTE(Reg); Reg = (Reg << 8) + *pInput++;*)");
                &Emit("        (*Act = pState->Action[Next8]; goto BitAlign$NewAlign; ... switch (Action) {*)");
                &Emit("");
                &Emit("        mov     al,[bx][00100H]", 2, 4);
                &Emit("        mov     dh,dl",           1, 2);
                &Emit("        mov     bl,dl",           1, 2);
                &Emit("        db 026H; (*es:*) mov dl,[si]",  3, 3);
                &Emit("        mov     ah,$NewAlign",    1, 2) if $Code != 8;
                &Emit("        inc     si",              1, 1);
                &Emit("        mov     bh,0",            1, 2);
                &Emit("        stosb",                   5, 1);
                &Emit("        mov     al,[bx]",         1, 2);
                &Emit("        jmp     ax",              5, 2);

        } else {
                &Emit("        (**pOutput++ = pStateTable[Next8 + NRBYTES];*)");
                &Emit("        (*pStateTable = &pTab->Table[0][0];*)");
                &Emit("        (*Next8 = LOBYTE(Reg >> $Shift);*)");
                &Emit("        (*Act = pState->Action[Next8]; goto BitAlign$NewAlign; ... switch (Action) {*)");
                &Emit("");
                &Emit("        mov     al,[bx][00100H]", 2, 4);
                &Emit("        mov     bx,dx",           1, 2);
                &Emit("        shr     bx,$Shift",       2, 3);
                &Emit("        mov     ah,$NewAlign",    1, 2) if $Code != 8;
                &Emit("        mov     bh,0",            1, 2);
                &Emit("        stosb",                   5, 1);
                &Emit("        mov     al,[bx]",         1, 2);
                &Emit("        jmp     ax",              5, 2);

        }

        &Emit("");

        &PadEntry($Code);

}


##########################################################################
# Emit -- Write instructions to output, and keep track of clocks and bytes
sub Emit {
        local($Text) = shift(@_);
        if (defined(@_)) {
                local($Clocks) = shift(@_);
                local($Bytes) = shift(@_);

                #Rewrite instruction if non-8086 (shift immediate)
                if ($Text =~ /^ +(shl|shr) +(al|ax|bl|bx),(\d+)/) {
                        if ($3 == 1) {
                                $Text = "        $1     $2,1";
                                $Clocks = 3;
                                $Bytes = 2;
                        } else {
                                $Text = "        ". $ShiftSubst{$1 . $2} . $3;
                        }
                }

                printf("%-40s (*%d, %d*)\n", $Text, $Clocks, $Bytes);

                $ClocksSum += $Clocks;
                $BytesSum  += $Bytes;

        } else {
            printf("%s\n", $Text);
        }

} #Emit

##########################################################################
# PadEntry -- Pad to next extry starting point, also report stats
sub PadEntry {
        local($EntryNr) = @_;

        $PC += $BytesSum;

        $PCNext = $PCNextArray[$EntryNr];

        printf("        (*%u cycles, %u bytes, PC 0x%04x, next PC 0x%04x*)\n",
                             $ClocksSum, $BytesSum,
                             $PC + ($BitAlign * 0x100),
                             $PCNext + ($BitAlign * 0x100));
        $ClocksSum = 0;
        $BytesSum = 0;

        if ($PC > $PCNext) {
                printf("*********** panic: $PC > $PCNext ************\n");
                exit;
        }

        while ($PC < $PCNext) {
                printf("        nop\n");
                $PC++;

        }

        if ($EntryNr == 0) {
                $PC = 0;
        }

        print "\n\n";

} #PadEntry

format ROUTINE_START=
(************************************************************************)
(*                                                                      *)
(*      ACodec -- Optimised Huffman decoder in assembly                 *)
(*                                                                      *)
(*      NOTE: This source file was created by the Perl script           *)
(*      mkfsma.pl.  If you want to change this source, do so            *)
(*      by editing the script and re-running Perl.                      *)
(*                                                                      *)
(*      Although this code only supports emitting 8-bit codes,          *)
(*      it's easy to output 16-bit codes by adding:                     *)
(*              mov ah,[bx][00200H]                                     *)
(*      after the mov al[...] instruction, and changing stosb           *)
(*      to stosw.  Each action has at least 4 spare bytes before        *)
(*      the next entry point to cater for this.  Some code would        *)
(*      need to be re-ordered as ah is loaded early to exploit the      *)
(*      Pentium's pipelined execution.  You will also need to change    *)
(*      the state table generation to add the high-byte tables.         *)
(*                                                                      *)
(*      WARNING: This code uses immediate shift instructions,           *)
(*      which are not supported on the 8086 and 8088.  This             *)
(*      instruction (supported by 80186 onwards) is shorter and         *)
(*      much faster than the portable method using count in CL.         *)
(*                                                                      *)
(*      Copyright (C) 1997 Grouse Software.  All rights reserved.       *)
(*      Written for Grouse by behoffski (Brenton Hoff).                 *)
(*                                                                      *)
(*      Free software: no warranty; use anywhere is ok; spread the      *)
(*      sources; note any mods; share variations and derivatives        *)
(*      (including sending to behoffski@grouse.com.au).                 *)
                                       "@"
(*                                                                      *)
(************************************************************************)

(*Entry point is at end so that code pages begin at offset 0x0000*)

(*
Inner loop register assignment:

        ah      bit alignment (7..0) for unpacking input stream
        bl      Next8 bits of input bitstream
        ds:bx   &pStateTable[Next8]; also note &pStateTable[Next8 + 256]
        dx      Current and next bytes of input stream
        es:si   pInput -- compressed bitstream (needs es: override)
        es:di   pOutput -- Values emitted by decoder

?? Assumes (Action;Values) array of tables is aligned to segment (bh == 0???)

Values after each instruction are (clocks, bytes).  Using 486 reference.
Pentium is much faster because of superscalar execution.

*)

        module ACodec

        segment _ACODEC_TEXT(CODE, 68H)

.

format ROUTINE_END=

(************************************************************************)
(*                                                                      *)
(*      Decode -- Interpret bitstream and write decoded symbols         *)
(*                                                                      *)
(*      void Decode(CCODEC_TABLE pTab, BYTE *pInput,                    *)
(*                  UINT *pNrDecodeBytes, BYTE *pOutput);               *)
(*                                                                      *)
(************************************************************************)

pOUTPUT_OFS                     = 6
pOUTPUT_SEG                     = 8
pNR_DECODE_BYTES_OFS            = 10
pNR_DECODE_BYTES_SEG            = 12

public  _ACodec_Decode:
        (*pTab in bx:ax*)
        (*pInput in dx:cx*)
        (*pNrDecodeBytes on stack*)
        (*pOutput on stack*)

        (*pTab must have offset of 0*)
        (*pOutput segment must match pInput*)

        (*Set up call frame*)
        push    bp
        mov     bp,sp

        (*Save registers used by this function*)
        push    si
        push    di
        push    ds
        push    es

        (*Save pInput offset for reporting bytes consumed*)
        push    ax

        (*pInput into es:si*)
        mov     es,dx
        mov     si,cx

        (*pOutput into es:di (es must match pInput)*)
        mov     di,[bp][pOUTPUT_OFS]

        (*pTab into ds:bx (but we require offset (bx) to be 0)*)
        mov     ds,bx
        mov     bh,0

        (*Prepare *pInput and reg held in dx and bl respectively*)
        db 026H; (*es:*) mov     dh,[si]
        inc     si
        db 026H; (*es:*) mov     dl,[si]
        inc     si
        mov     bl,dh

        (*Start with bit alignment of 7*)
        mov     ah,7

        (*Dispatch control to the action within the starting code page*)
        mov     al,[bx]
        jmp     ax


        (*Handle end-of-buffer actions*)
Finished:
        (*Get remaining bit count of stop-decoding symbol*)
        mov     al,[bx][00100H]

        (*Did we consume bits without needing the lookahead byte?*)
        cmp     ah,al
        jb      NextUsed

        (*Next byte wasn't used: don't consume it*)
        dec     si

NextUsed:
        (*Report how many bytes were emitted by the decoder*)
        mov     bx,[bp][pOUTPUT_OFS]
        sub     di,bx
        lds     bx,[bp][pNR_DECODE_BYTES_OFS]
        mov     [bx],di

        (*Calculate and return number of input bytes used by decoder*)
        pop     bx
        sub     si,bx
        mov     ax,si
        inc     ax

        (*Restore registers*)
        pop     es
        pop     ds
        pop     di
        pop     si

        pop     bp

        ret     far 8

        end
.
