##########################################################################
# MkFSMC -- Make finite-state machine, C version
#
# There are 8 bit alingment states, and 10 actions per state.
# Creating the finite-state machine code is mechanical and
# tedious.  This Perl script creates the source in a more
# creative fashion.
#
# Copyright (C) Grouse Software 1995-7.  All rights reserved.
# Written for Grouse by behoffski (Brenton Hoff).
#
# Free software: no warranty; use anywhere is ok; spread the
# sources; note any mods; share variations and derivatives
# (including sending to behoffski@grouse.com.au).

$~ = ROUTINE_START;
write STDOUT;

$Debug = 0;

for ($BitAlign = 7; $BitAlign >= 0; $BitAlign--) {
        #Write header for this state
        print "BitAlign$BitAlign:\n";
        print "        if (NrBytes-- == 0) goto LastAct;\n";
#        print "        if ((NrBytes < 200) || ((pOutput - pOutStart) < 200)) {\n"        if $Debug;
        print "                CCodec_Trace(Act, pOutput[-1], Next8, Reg, $BitAlign, \n" if $Debug;
        print "                                  pInput[0], pInput[1]);\n"               if $Debug;
#        print "        }\n"                                                              if $Debug;
        print "        switch (Act) {\n";

        for ($Code = 1; $Code < 9; $Code++) {
                &MkEntry($BitAlign, $Code);
        }

        $Shift = $BitAlign + 1;
        print "        case USEGT8:\n";
        print "                NrBytes++;\n";
        print "                pStateTable = &pTab->Table[pStateTable[Next8 + NRBYTES]][0];\n";
        if ($Shift != 8) {
                print "                Reg = (Reg << 8) | *pInput++;\n";
                print "                Next8 = LOBYTE(Reg >> $Shift);\n";
                print "                Act = pStateTable[Next8];\n";
                print "                goto BitAlign$BitAlign; /*break;*/\n";
        } else {
                print "                Next8 = LOBYTE(Reg);\n";
                print "                Reg = (Reg << 8) | *pInput++;\n";
                print "                Act = pStateTable[Next8];\n";
                print "                goto BitAlign$BitAlign; /*break;*/\n";
        }
        print "\n";
        print "        case STOP:\n";
        print "                LastAlign = $BitAlign;\n";
        print "                goto LastAct;\n";
        print "\n";
        print "        default:\n";
        print "                printf(\"?? Unknown action: %02x\\n\", Act);\n";
        print "                exit(62);\n";
        print "\n";
        print "        }\n";
        print "\n";

}

$~ = ROUTINE_END;
write STDOUT;

exit 1;

##########################################################################
# MkEntry -- Create entry for specified bit alignment and code length
sub MkEntry {
        local($BitAlign, $Code) = @_;
        local($NewAlign);
        local($Shift);

        print "        case USE$Code:\n";
        print "                *pOutput++ = pStateTable[Next8 + NRBYTES];\n";
        print "                pStateTable = &pTab->Table[0][0];\n";

        if ($BitAlign < $Code) {
                $NewAlign = $BitAlign + (8 - $Code);
                $Shift = $NewAlign + 1;
                if ($Shift != 8) {
                        print "                Reg = (Reg << 8) | *pInput++;\n";
                        print "                Next8 = LOBYTE(Reg >> $Shift);\n";
                        print "                Act = pStateTable[Next8];\n";
                        print "                goto BitAlign$NewAlign; /*break;*/\n";
                } else {
                        print "                Next8 = LOBYTE(Reg);\n";
                        print "                Reg = (Reg << 8) | *pInput++;\n";
                        print "                Act = pStateTable[Next8];\n";
                        print "                goto BitAlign$NewAlign; /*break;*/\n";
                }
        } else {
                $NewAlign = $BitAlign - $Code;
                $Shift = $NewAlign + 1;
                print "                Next8 = LOBYTE(Reg >> $Shift);\n";
                print "                Act = pStateTable[Next8];\n";
                print "                goto BitAlign$NewAlign; /*break;*/\n";
        }
        print "\n";

}


format ROUTINE_START=
/************************************************************************/
/*                                                                      */
/*      Decode -- Decode compressed stream using table-driven FSA       */
/*                                                                      */
/************************************************************************/
UINT
CCodec_Decode(CCODEC_TABLE pTab, BYTE *pInput,
              UINT *pNrDecodeBytes, BYTE *pOutput)
{
        BYTE *pInStart = pInput;
        BYTE *pOutStart = pOutput;
        WORD Reg = BYTES2WORD(pInput[0], pInput[1]);
        UINT Act = STOP;
        BYTE *pStateTable = &pTab->Table[0][0];
        BYTE LastAlign;
        BYTE Next8;
        UINT NrBytes = 32767;

        pInput += 2;
        Next8 = HIBYTE(Reg);
        Act = pStateTable[Next8];
.

format ROUTINE_END=

LastAct:
        /*Did decoding the stop symbol stay within the current byte?*/
        if (LastAlign >= pStateTable[Next8 + NRBYTES]) {
                /*Yes, don't consume next byte*/
                pInput--;
        }

#        printf("\nLastAlign, pStateTable, Value: %u, %p, %u\n",
#                           LastAlign, pStateTable, pStateTable[Next8 + NRBYTES]);
#        CCodec_Trace(Act, pOutput[-1], Next8, Reg, 0, pInput[0], pInput[1]);

        *pNrDecodeBytes = pOutput - pOutStart;
        return pInput - pInStart + 1;

#pragma warn(wpnu => off)
} /*Decode*/

.
