/************************************************************************/
/*                                                                      */
/*      wc -- Count words, lines and bytes of (text) files              */
/*                                                                      */
/*      Copyright (C) Grouse Software 1995-7.  All rights reserved.     */
/*      Written for Grouse by behoffski (Brenton Hoff).                 */
/*                                                                      */
/*      Free software: no warranty; use anywhere is ok; spread the      */
/*      sources; note any mods; share variations and derivatives        */
/*      (including sending to behoffski@grouse.com.au).                 */
/*                                                                      */
/************************************************************************/

#ifndef WC_H
#define WC_H

#include <compdef.h>

#ifdef __cplusplus
extern "c" {
#endif /*__cplusplus*/

/*Prototype for "read" function (ANSI-compatible)*/
typedef int (Wc_ReadFn)(int Handle, void *pBuf, unsigned NrBytes);

/************************************************************************/
/*                                                                      */
/*      Init -- Prepare module for operation                            */
/*                                                                      */
/************************************************************************/
void
Wc_Init(void);

/************************************************************************/
/*                                                                      */
/*      Start -- Begin managing what has to be managed                  */
/*                                                                      */
/************************************************************************/
void
Wc_Start(void);

/************************************************************************/
/*                                                                      */
/*      main -- Control operation of program                            */
/*                                                                      */
/************************************************************************/
int
Wc_main(int argc, char **argv);

/************************************************************************/
/*                                                                      */
/*      ReadFunction -- Receive function to read bytes from a file      */
/*                                                                      */
/*      wc assumes that the ANSI read function is sufficient            */
/*      and correct to fetch file lines.  If this assumption is         */
/*      incorrect for some reason, the client may supply an             */
/*      alternative function via this interface.                        */
/*                                                                      */
/*      This interface was included when behoffski found that           */
/*      the operation of read for pipes was incorrect,                  */
/*      whereas _read worked correctly under TopSpeed C v3.10.          */
/*                                                                      */
/************************************************************************/
void
Wc_ReadFunction(Wc_ReadFn *pReadFn);

#ifdef __cplusplus
}
#endif /*__cplusplus*/

#endif /*WC_H*/
