/************************************************************************/
/*                                                                      */
/*      MatchC -- ANSI C version of match engine                        */
/*                                                                      */
/*      This module implements the table-driven search using ANSI C     */
/*      constructs.  It is invaluable for debugging state table         */
/*      generation and for prototyping match engine actions.            */
/*      The performance will usually be very good for a C version,      */
/*      but will not come close to an implementation exploiting         */
/*      the Grouse Finite State Architecture in assembly.               */
/*                                                                      */
/*      Copyright (C) Grouse Software 1995-7.  All rights reserved.     */
/*      Written for Grouse by behoffski (Brenton Hoff).                 */
/*                                                                      */
/*      Free software: no warranty; use anywhere is ok; spread the      */
/*      sources; note any mods; share variations and derivatives        */
/*      (including sending to behoffski@grouse.com.au).                 */
/*                                                                      */
/************************************************************************/

#ifndef MATCHC_H
#define MATCHC_H

#include <compdef.h>
#include "matcheng.h"

#ifdef __cplusplus
extern "C" {
#endif /*__cplusplus*/

/*Debugging options*/
#define MATCHC_D_CONTROL                BIT0
#define MATCHC_D_ACTION                 BIT1

/************************************************************************/
/*                                                                      */
/*      Init -- Prepare module for operation                            */
/*                                                                      */
/************************************************************************/
void
MatchC_Init(void);

/************************************************************************/
/*                                                                      */
/*      Match -- Perform search and report success                      */
/*                                                                      */
/************************************************************************/
BOOL PAGE_ALIGNED
MatchC_Match(MATCHENG_SPEC *pSpec, CHAR *pTextStart,
                MATCHENG_DETAILS *pDetails);

/************************************************************************/
/*                                                                      */
/*      Debug -- Set debugging options                                  */
/*                                                                      */
/************************************************************************/
void
MatchC_Debug(LWORD OffBits, LWORD OnBits);

#ifdef __cplusplus
}
#endif /*__cplusplus*/

#endif /*MATCHC_H*/
